@@ -55,6 +55,11 @@
 #' model have an `epoch` argument (which defaults to the epoch with the best
 #' loss value).
 #'
+#' The use of the L1 penalty (a.k.a. the lasso penalty) does _not_ force
+#' parameters to be strictly zero (as it does in packages such as \pkg{glmnet}).
+#' The zeroing out of parameters is a specific feature the optimization method
+#' used in those packages.
+#'
 #' @seealso [predict.brulee_linear_reg()], [coef.brulee_linear_reg()],
 #' [autoplot.brulee_linear_reg()]
 #'
@@ -156,6 +161,7 @@ brulee_linear_reg.data.frame <-
            y,
            epochs = 20L,
            penalty = 0.001,
+           mixture = 0,
            validation = 0.1,
            optimizer = "LBFGS",
            learn_rate = 1.0,
@@ -172,6 +178,7 @@ brulee_linear_reg.data.frame <-
       optimizer = optimizer,
       learn_rate = learn_rate,
       penalty = penalty,
+      mixture = mixture,
       validation = validation,
       momentum = momentum,
       batch_size = batch_size,
@@ -189,6 +196,7 @@ brulee_linear_reg.matrix <- function(x,
                                      y,
                                      epochs = 20L,
                                      penalty = 0.001,
+                                     mixture = 0,
                                      validation = 0.1,
                                      optimizer = "LBFGS",
                                      learn_rate = 1,
@@ -206,6 +214,7 @@ brulee_linear_reg.matrix <- function(x,
     learn_rate = learn_rate,
     momentum = momentum,
     penalty = penalty,
+    mixture = mixture,
     validation = validation,
     batch_size = batch_size,
     stop_iter = stop_iter,
@@ -223,6 +232,7 @@ brulee_linear_reg.formula <-
            data,
            epochs = 20L,
            penalty = 0.001,
+           mixture = 0,
            validation = 0.1,
            optimizer = "LBFGS",
            learn_rate = 1,
@@ -240,6 +250,7 @@ brulee_linear_reg.formula <-
       learn_rate = learn_rate,
       momentum = momentum,
       penalty = penalty,
+      mixture = mixture,
       validation = validation,
       batch_size = batch_size,
       stop_iter = stop_iter,
@@ -257,6 +268,7 @@ brulee_linear_reg.recipe <-
            data,
            epochs = 20L,
            penalty = 0.001,
+           mixture = 0,
            validation = 0.1,
            optimizer = "LBFGS",
            learn_rate = 1,
@@ -274,6 +286,7 @@ brulee_linear_reg.recipe <-
       learn_rate = learn_rate,
       momentum = momentum,
       penalty = penalty,
+      mixture = mixture,
       validation = validation,
       batch_size = batch_size,
       stop_iter = stop_iter,
@@ -286,7 +299,7 @@ brulee_linear_reg.recipe <-
 # Bridge
 
 brulee_linear_reg_bridge <- function(processed, epochs, optimizer,
-                                     learn_rate, momentum, penalty, dropout,
+                                     learn_rate, momentum, penalty, mixture, dropout,
                                      validation, batch_size, stop_iter, verbose, ...) {
   if(!torch::torch_is_installed()) {
     rlang::abort("The torch backend has not been installed; use `torch::install_torch()`.")
@@ -306,6 +319,7 @@ brulee_linear_reg_bridge <- function(processed, epochs, optimizer,
     check_integer(batch_size, single = TRUE, 1, fn = f_nm)
   }
   check_double(penalty, single = TRUE, 0, incl = c(TRUE, TRUE), fn = f_nm)
+  check_double(mixture, single = TRUE, 0, 1, incl = c(TRUE, TRUE), fn = f_nm)
   check_double(validation, single = TRUE, 0, 1, incl = c(TRUE, FALSE), fn = f_nm)
   check_double(momentum, single = TRUE, 0, 1, incl = c(TRUE, TRUE), fn = f_nm)
   check_double(learn_rate, single = TRUE, 0, incl = c(FALSE, TRUE), fn = f_nm)
@@ -342,6 +356,7 @@ brulee_linear_reg_bridge <- function(processed, epochs, optimizer,
       learn_rate = learn_rate,
       momentum = momentum,
       penalty = penalty,
+      mixture = mixture,
       validation = validation,
       batch_size = batch_size,
       stop_iter = stop_iter,
@@ -399,6 +414,7 @@ linear_reg_fit_imp <-
            epochs = 20L,
            batch_size = 32,
            penalty = 0.001,
+           mixture = 0,
            validation = 0.1,
            optimizer = "LBFGS",
            learn_rate = 1,
@@ -462,6 +478,7 @@ linear_reg_fit_imp <-
     ## ---------------------------------------------------------------------------
     # Initialize model and optimizer
     model <- linear_reg_module(ncol(x))
+    loss_fn <- make_penalized_loss(loss_fn, model, penalty, mixture)
 
     # Write a optim wrapper
     if (optimizer == "LBFGS") {
@@ -469,8 +486,7 @@ linear_reg_fit_imp <-
                                       history_size = 5)
     } else if (optimizer == "SGD") {
       optimizer <-
-        torch::optim_sgd(model$parameters, lr = learn_rate,
-                         weight_decay = penalty, momentum = momentum)
+        torch::optim_sgd(model$parameters, lr = learn_rate, momentum = momentum)
     } else {
       rlang::abort(paste0("Unknown optimizer '", optimizer, "'"))
     }
@@ -563,7 +579,8 @@ linear_reg_fit_imp <-
       dims = list(p = p, n = n, h = 0, y = y_dim, features = colnames(x)),
       y_stats = y_stats,
       parameters = list(learn_rate = learn_rate,
-                        penalty = penalty, validation = validation,
+                        penalty = penalty, mixture = mixture,
+                        validation = validation,
                         batch_size = batch_size, momentum = momentum)
     )
   }
@@ -587,3 +604,4 @@ print.brulee_linear_reg <- function(x, ...) {
   cat("Linear regression\n\n")
   brulee_print(x)
 }
+
